package com.jwipc.order.dto;

import com.jwipc.api.dto.MesBaseParamDTO;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;
import java.util.List;

/**
 * <p> </p>
 *
 * @author ZhangJie
 * @version 1.0
 * @since JDK 1.8
 */
@Data
public class DeliveryOrderToMesDTO extends MesBaseParamDTO implements Serializable {
    @Serial
    private static final long serialVersionUID = 8565660438495796736L;
    private IT_HEAD IT_HEAD;
    private IT_ITEM IT_ITEM;
    @Data
    public static class IT_HEAD {
        private List<IT_HEAD_Item> item;
        // 省略构造函数和 getter/setter 方法
    }
    @Data
    public static class IT_HEAD_Item {
        private String RCV_NO;
        private String Vcode;
        private String Vname;
        private String Buyer;
        private String DeliveryTime;
        private String DeliveryTimeHH;
        private String ReceivingAddress;
        private String DeliveryMan;
        private String Contract_No;

        // 省略构造函数和 getter/setter 方法
    }
    @Data
    public static class IT_ITEM {
        private List<IT_ITEM_Item> item;

        // 省略构造函数和 getter/setter 方法
    }
    @Data
    public static class IT_ITEM_Item {
        private String RCV_NO;
        private String PO_No;
        private String PO_Line;
        private String PartNo;
        private String Description;
        private String MaterialType;
        private String Unit;
        private String Price;
        private String NeedTime;
        private String PO_Qty;
        private String RCVD_Qty;
        private String SHP_Qty;
        private String VendorCode;
        private String VendorName;
        private String State;
        private String Reserve1;

        // 省略构造函数和 getter/setter 方法
    }
}
